@@ -288,7 +288,7 @@ def live_server(request):
     --liveserver command line option or if this is not provided from
     the DJANGO_LIVE_TEST_SERVER_ADDRESS environment variable.  If
     neither is provided ``localhost:8081,8100-8200`` is used.  See the
-    Django documentation for it's full syntax.
+    Django documentation for its full syntax.
 
     NOTE: If the live server needs database access to handle a request
           your test will have to request database access.  Furthermore
@@ -340,9 +340,17 @@ def _live_server_helper(request):
     The separate helper is required since live_server can not request
     transactional_db directly since it is session scoped instead of
     function-scoped.
+
+    It will also override settings only for the duration of the test.
     """
-    if 'live_server' in request.funcargnames:
-        getfixturevalue(request, 'transactional_db')
+    if 'live_server' not in request.funcargnames:
+        return
+
+    getfixturevalue(request, 'transactional_db')
+
+    live_server = getfixturevalue(request, 'live_server')
+    live_server._live_server_modified_settings.enable()
+    request.addfinalizer(live_server._live_server_modified_settings.disable)
 
 
 @pytest.fixture(scope='function')
