@@ -277,7 +277,9 @@ def _classmethod_is_defined_at_leaf(cls, method_name):
         f = method.__func__
     except AttributeError:
         pytest.fail('%s.%s should be a classmethod' % (cls, method_name))
-    if PY2 and not (inspect.ismethod(method) and method.__self__ is cls):
+    if PY2 and not (inspect.ismethod(method) and
+                    inspect.isclass(method.__self__) and
+                    issubclass(cls, method.__self__)):
         pytest.fail('%s.%s should be a classmethod' % (cls, method_name))
     return f is not super_method.__func__
 
@@ -290,9 +292,11 @@ def _disable_class_methods(cls):
         return
 
     _disabled_classmethods[cls] = (
-        cls.setUpClass,
+        # Get the classmethod object (not the resulting bound method),
+        # otherwise inheritence will be broken when restoring.
+        cls.__dict__.get('setUpClass'),
         _classmethod_is_defined_at_leaf(cls, 'setUpClass'),
-        cls.tearDownClass,
+        cls.__dict__.get('tearDownClass'),
         _classmethod_is_defined_at_leaf(cls, 'tearDownClass'),
     )
 
@@ -378,10 +382,10 @@ def _django_db_marker(request):
     This will dynamically request the ``db`` or ``transactional_db``
     fixtures as required by the django_db marker.
     """
-    marker = request.keywords.get('django_db', None)
+    marker = request.node.get_closest_marker('django_db')
     if marker:
-        validate_django_db(marker)
-        if marker.transaction:
+        transaction = validate_django_db(marker)
+        if transaction:
             getfixturevalue(request, 'transactional_db')
         else:
             getfixturevalue(request, 'db')
@@ -436,7 +440,7 @@ def _dj_autoclear_mailbox():
 
 
 @pytest.fixture(scope='function')
-def mailoutbox(monkeypatch, _dj_autoclear_mailbox):
+def mailoutbox(monkeypatch, django_mail_patch_dns, _dj_autoclear_mailbox):
     if not django_settings_is_configured():
         return
 
@@ -444,10 +448,21 @@ def mailoutbox(monkeypatch, _dj_autoclear_mailbox):
     return mail.outbox
 
 
+@pytest.fixture(scope='function')
+def django_mail_patch_dns(monkeypatch, django_mail_dnsname):
+    from django.core import mail
+    monkeypatch.setattr(mail.message, 'DNS_NAME', django_mail_dnsname)
+
+
+@pytest.fixture(scope='function')
+def django_mail_dnsname(monkeypatch):
+    return 'fake-tests.example.com'
+
+
 @pytest.fixture(autouse=True, scope='function')
 def _django_set_urlconf(request):
     """Apply the @pytest.mark.urls marker, internal to pytest-django."""
-    marker = request.keywords.get('urls', None)
+    marker = request.node.get_closest_marker('urls')
     if marker:
         skip_if_no_django()
         import django.conf
@@ -457,9 +472,9 @@ def _django_set_urlconf(request):
             # Removed in Django 2.0
             from django.core.urlresolvers import clear_url_caches, set_urlconf
 
-        validate_urls(marker)
+        urls = validate_urls(marker)
         original_urlconf = django.conf.settings.ROOT_URLCONF
-        django.conf.settings.ROOT_URLCONF = marker.urls
+        django.conf.settings.ROOT_URLCONF = urls
         clear_url_caches()
         set_urlconf(None)
 
@@ -656,8 +671,8 @@ def validate_django_db(marker):
     the marker which will have the correct value.
     """
     def apifun(transaction=False):
-        marker.transaction = transaction
-    apifun(*marker.args, **marker.kwargs)
+        return transaction
+    return apifun(*marker.args, **marker.kwargs)
 
 
 def validate_urls(marker):
@@ -667,5 +682,5 @@ def validate_urls(marker):
     marker which will have the correct value.
     """
     def apifun(urls):
-        marker.urls = urls
-    apifun(*marker.args, **marker.kwargs)
+        return urls
+    return apifun(*marker.args, **marker.kwargs)
